@@ -28,8 +28,8 @@ def delete_from_tar_file(file, files_to_delete):
         os.remove(os.path.join(temp_dir, delete_file))
 
     with tarfile.open(file, 'w') as f:
-        for file_to_add in pathlib.Path(temp_dir).rglob('*'):
-            f.add(file_to_add, file_to_add.relative_to(temp_dir))
+        for file_to_add in pathlib.Path(temp_dir).glob('*'):
+            f.add(file_to_add, file_to_add.relative_to(temp_dir), recursive=True)
     shutil.rmtree(temp_dir)
 
 
@@ -93,9 +93,11 @@ def slimify(save_image_path, base_image, cache_dir=None):
         manifest_file = f.extractfile(f.getmember('manifest.json'))
         manifest = json.load(manifest_file)
         layer_entry_map = {}
-        duplicate_use_layers = set()
+        # we are not allowed to remove layers that appear multiple times in an image
+        forbidden_layers = set()
 
         for image_information in manifest:
+            seen_digests = set()
             image_layer_entries = image_information['Layers']
             config_name = image_information['Config']
             config_file = f.extractfile(f.getmember(config_name))
@@ -103,18 +105,12 @@ def slimify(save_image_path, base_image, cache_dir=None):
             assert(config['architecture'] == 'arm64')
             image_layers = config['rootfs']['diff_ids']
             for digest, file in zip(image_layers, image_layer_entries):
-                if digest in duplicate_use_layers:
-                    continue
-                if digest in layer_entry_map:
-                    # we are not allowed to strip dual-used layers, as docker might try to reference
-                    # such a layer if used in user code. Most likely empty layer.
-                    del layer_entry_map[digest]
-                    duplicate_use_layers.add(digest)
-                    continue
-                
+                if digest in seen_digests:
+                    forbidden_layers.add(digest)
+                seen_digests.add(digest)
                 layer_entry_map[digest] = (file, file in f.getnames())
 
-    removable_layers = set(layer_entry_map.keys()).intersection(set(base_layers))
+    removable_layers = set(layer_entry_map.keys()).intersection(set(base_layers)).difference(forbidden_layers)
     print(f'> Will remove {len(removable_layers)} from app image')
     for l in removable_layers:
         entry, exists = layer_entry_map[l]
